/*****************************************************************************
 * HRegion.h
 ***************************************************************************** 
 *
 * Project:     Halcon/C++
 * Description: Types for regions
 *
 * (c) 1996-2000 by MVTec Software GmbH
 *                  Neherstr. 1, 81675 Muenchen, Germany
 * 
 *****************************************************************************
 *
 * $Revision: 1.15 $
 * $Date: 2001/12/06 09:55:22 $
 *
 */


#ifndef H_REGION_H
#define H_REGION_H

#include "Hconst.h"
#include "IPtype.h"


/****************************************************************************/
/*                               HRegion                                    */
/****************************************************************************/

// <summary>
// Klasse zur Bearbeitung von Regionen.
// </summary>

// <prerequisite>
//    <li> HAbstractRegion
// </prerequisite>

// <etymology>
// HRegion ist die Kurzform fr Halcon-Region.
// </etymology>

// <reviewed reviewer="" date="" tests="">

// <synopsis>
// Als Basistyp fr Bildobjekte verwendet Halcon die virtuelle Klasse 
// HObject. Von dieser Klasse werden drei weitere Klassen abgeleitet:
//    <li> Bilder   (HImage)
//    <li> Polygone (HXLD)
//    <li> Regionen (HRegion)<BR>
// Eine Region besteht aus einer Menge von Koordinaten in der Bildebene.
// Eine solche Region mu durchaus nicht zusammenhngend sein und kann
// ohne weiteres auch "Lcher" enthalten. Regionen knnen auch grer als
// das aktuelle Bildformat sein. Intern werden Regionen durch 
// Lauflngenkodierung realisiert. Die zugehrige Klasse HRegion enthlt
// alle wichtigen Operationen zur Bearbeitung von Regionen.
// </synopsis>

// <motivation>
// HRegion bietet eine Zusammenfassung aller Operationen, die der Bearbeitung
// von Regionen dienen.
// </motivation>

// <use visibility=export>

class HRegionArray;
class LIntExport HRegion: public HAbstractRegion, public HObject {
  friend class HRegionArray;
public:
  // Default-Konstruktor. Er erzeugt eine leere Region, d.h. die Flche 
  // ist Null. Nicht alle Prozeduren (z.B. manche Formmerkmale) lassen 
  // sich auf diese Region anwenden.
  HRegion(void): HObject(), rl(0) {num_regions++;}
  //
  HRegion(Hkey key): HObject(key), rl(0) {num_regions++;}
  //
  HRegion(const Hobject &obj);
  // Erzeugung einer Region aus einer Sehne.
  // Eine Sehne ist eine horizontale Linie. 
  HRegion(const HDChord &line);
  // Erzeugung einer Region aus einem diskreten 2-dimensionalen Punkt.
  HRegion(const HDPoint2D &point);
  // Erzeugung einer Region aus einem Rechteck parallel zu den 
  // Koordinatenachsen mit kontinuierlichen Koordinaten.
  HRegion(const HRectangle1 &obj);
  // Erzeugung einer Region aus einem beliebig orientierten Rechteck mit
  // kontinuierlichen Koordinaten.
  HRegion(const HRectangle2 &obj);
  // Erzeugung einer Region aus einem Kreis mit kontinuierlichen Koordinaten.
  HRegion(const HCircle &obj);
  // Erzeugung einer Region aus einer beiliebig orientierten Ellipse mit
  // kontinuierlichen Koordinaten.
  HRegion(const HEllipse &obj);
  // Erzeugung einer Region durch Einlesen einer Datei. Die Datei kann mit
  // der Member-Funktion WriteRegion erzeugt werden.
  HRegion(const char *file);
  //
  HRegion(const HAbstractRegion *base);
  // Copy-Konstruktor.
  HRegion(const HRegion &reg);
  HRegion(const HRegionArray &arr);
  // Destruktor. Im Gegensatz zu den primitiven Halcon-Objekten (Hkey) 
  // braucht man sich bei dieser Klasse nicht um die Freigabe des
  // Speichers kmmern.
  ~HRegion(void)       {num_regions--;}
  // Zuweisungsoperator.
  HRegion &operator = (const HRegion &reg);
  HRegion &operator = (const HRegionArray &arr);

  /*----------------------- Tools ------------------------------------------*/
  //
  void          Transform(const HAffineTrans &transform, int mode);
  // Ausgabe der Region in dem angegebenen Fenster.
  virtual       void Display(const HWindow &w) const;
  // Name der Halcon-Klasse -> get_obj_class
  virtual const char *HClassName(void) const;
  //
  virtual const char *ClassName(void) const;
  //
  virtual const char *InstClassName(void) const;
  //
  virtual       INT4 ClassDBID(void) const {return REGION_ID;}
  //
  virtual       void CheckMyObjClass(void) const;
  //
  HBool          Shared(void) const;

  Hrlregion      *GetRL(void) const;

  /*----------------------- Infixoperationen -------------------------------*/ 
  // Grenskalierung der Region um den angegebenen Faktor. Das Zentrum der
  // Skalierung ist der Ursprung (0,0).
  HRegion  operator * (double scale) const;

  // Minkowsky-Subtraktion der Region mit einem Kreis mit Radius 'radius'.
  // <group>
  HRegion  operator >> (double radius) const;
  HRegion &operator >>= (double radius);
  //</group>

  // Minkowsky-Addition der Region mit einem Kreis mit Radius 'radius'.
  // <group>
  HRegion  operator << (double radius) const;
  HRegion &operator <<= (double radius);
  //</group>

  // Translation der Region mit einem diskreten Punkt.
  // <group>
  HRegion  operator + (const HDPoint2D &point) const;
  HRegion &operator += (const HDPoint2D &point);
  //</group>

  // Minkowsky-Addition der Region mit einem Kreuz, das aus insgesamt
  // 5 Punkten besteht.
  HRegion &operator ++ (void);

  // Minkowsky-Addition der Region mit einer anderen Region.
  // <group>
  HRegion  operator + (const HRegion &reg) const;
  HRegion &operator += (const HRegion &reg);
  //</group>

  // Minkowsky-Subtraktion der Region mit einer anderen Region.
  // <group>
  HRegion  operator - (const HRegion &reg) const;
  HRegion &operator -= (const HRegion &reg);
  //</group>

  // Minkowsky-Subtraktion der Region mit einem Kreuz, das aus insgesamt
  // 5 Punkten besteht.
  HRegion &operator -- (void);
  // Komplement der Region.
  HRegion  operator ~ (void) const;
  // Punktespiegelung der Region am Ursprung.
  HRegion  operator ! (void) const;

  // Durchschnitt der Region mit einer anderen Region.
  // <group>
  HRegion  operator & (const HRegion &reg) const;
  HRegion &operator &= (const HRegion &reg);
  //</group>

  // Vereinigung einer Region mit einer anderen Region.
  // <group>
  HRegion  operator | (const HRegion &reg) const;
  HRegion &operator |= (const HRegion &reg);
  //</group>

  // Von der Region eine andere Region abziehen.
  // <group>
  HRegion  operator / (const HRegion &reg) const;
  HRegion &operator /= (const HRegion &reg);
  // </group>

  // Vergleich zweier Regionen auf Identitt.
  HBool     operator == (const HRegion &reg) const;

  // Test auf Enthaltensein (Teilmengenbeziehung der beinhalteten Koordinaten).
  // <group>
  HBool     operator >= (const HRegion &reg) const;
  HBool     operator > (const HRegion &reg) const;
  HBool     operator <= (const HRegion &reg) const;
  HBool     operator < (const HRegion &reg) const;
  // </group>

  // new und delete
  // <group>
#ifdef WIN32EXTRA
  void*    operator new(size_t nSize);
  void     operator delete (void* p);
#endif
  // </group>
  /*----------------------- Merkmale, die in Halcon keine Funktionen sind ---*/
  // Orientierung der Region (Winkel der quivalenten Ellipse im Bogenma).
  double Phi(void) const;
  // Lnge des Hauptradius der quivalenten Ellipse.
  double Ra(void) const;
  // Lnge des Nebenradius der quivalenten Ellipse.
  double Rb(void) const;

  // Formfaktoren.
  // <group>
  double Anisometry(void) const;
  double Bulkiness(void) const;
  double StructureFactor(void) const;
  // </group>
  // Flche der Kontur.
  long   Area(void) const;

  // Schwerpunkt der Region.
  // <group>
  double X(void) const {return Col();}
  double Y(void) const {return Row();}
  // </group>

  //
  double Row(void) const;
  //
  double Col(void) const;

  // Momente der Region (siehe Halcon-Referenzhandbuch).
  // <group>
  double M11(void) const;
  double M20(void) const;
  double M02(void) const;
  double Ia(void) const;
  double Ib(void) const;
  // </group>
  
  // Klassen-Operationen.
  // <group>
  static long NumInst(void) { return num_regions; }
  static long NumData(void);
  // </group>

  /*----------------------- Funktionen fr HAbstractRegion ------------------*/
  // Erzeugen einer Kopie.
  HAbstractRegion *CopyAbstractRegion(void) const;
  // Erzeugen einer analogen leeren Kopie.
  HAbstractRegion *NewEmpty(void) const;
  //
  HRectangle1 BoundingBox(void) const;
  //
  HRectangle1 SmallestRectangle1(void) const;
  // berprfung, ob ein Punkt enthalten ist.
  HBool           In(const HDPoint2D &p) const;
  // Einfgen eines Punktes.
  void           Insert(const HDPoint2D &p);
  // Lschen eines Punktes.
  void           Delete(const HDPoint2D &p);
  //
  HBool           IsEmpty(void) const;
  //
  HAbstractRegion *ToRLRegion(void) const;
  //
  HAbstractRegion *ToHRegion(void) const;


private:
  static long num_regions;
  //	 
  Hrlregion  *rl;

  void  setRLptr(void) const;

public:
  // Extract the data values of the elements (in ECC 200: ``modules'') inside a
  // bar code region (``Data Matrix symbol'') and their positions in the image.
  virtual HTuple Get2dBarCodePos(const HImage &Image, const HTuple &BarCodeDescr, const HTuple &CodeRegDescr, const HTuple &GenParamNames, const HTuple &GenParamValues, HTuple *BarCodeData, HTuple *DataElementRow, HTuple *DataElementCol) const;
  // Extract the values of the data elements (in ECC 200: ``modules'') inside a
  // bar code region (``Data Matrix symbol'').
  virtual HTuple Get2dBarCode(const HImage &Image, const HTuple &BarCodeDescr, const HTuple &CodeRegDescr, const HTuple &GenParamNames, const HTuple &GenParamValues, HTuple *BarCodeData) const;
  // Grab and segment an image from the specified frame grabber and
  // start the asynchronous grab of the next image.
  static HRegion GrabRegionAsync(const HTuple &FGHandle, const HTuple &MaxDelay);
  static HRegion GrabRegionAsync(long FGHandle, double MaxDelay);
  // Grab and segment an image from the specified frame grabber.
  static HRegion GrabRegion(const HTuple &FGHandle);
  static HRegion GrabRegion(long FGHandle);
  // Add gray values to regions.
  virtual HImage AddChannels(const HImage &Image) const;
  // Write regions on file.
  virtual void WriteRegion(const HTuple &FileName) const;
  virtual void WriteRegion(const char *FileName) const;
  // Read binary images or Halcon regions.
  static HRegion ReadRegion(const HTuple &FileName);
  static HRegion ReadRegion(const char *FileName);
  // Close edge gaps using the edge amplitude image.
  virtual HRegion CloseEdgesLength(const HImage &Gradient, const HTuple &MinAmplitude, const HTuple &MaxGapLength) const;
  virtual HRegion CloseEdgesLength(const HImage &Gradient, long MinAmplitude, long MaxGapLength) const;
  // Close edge gaps using the edge amplitude image.
  virtual HRegion CloseEdges(const HImage &EdgeImage, const HTuple &MinAmplitude) const;
  virtual HRegion CloseEdges(const HImage &EdgeImage, long MinAmplitude) const;
  // Determine the noise distribution of an image.
  virtual HTuple NoiseDistributionMean(const HImage &Image, const HTuple &FilterSize) const;
  virtual HTuple NoiseDistributionMean(const HImage &Image, long FilterSize) const;
  // Determine the fuzzy entropy of regions.
  virtual double FuzzyEntropy(const HImage &Image, const HTuple &Apar, const HTuple &Cpar) const;
  virtual double FuzzyEntropy(const HImage &Image, long Apar, long Cpar) const;
  // Calculate the fuzzy perimeter of a region.
  virtual double FuzzyPerimeter(const HImage &Image, const HTuple &Apar, const HTuple &Cpar) const;
  virtual double FuzzyPerimeter(const HImage &Image, long Apar, long Cpar) const;
  // Paint regions with their average gray value.
  virtual HImage RegionToMean(const HImage &Image) const;
  // Displays regions in a window.
  virtual void DispRegion(const HTuple &WindowHandle) const;
  virtual void DispRegion(long WindowHandle) const;
  // Query the icon for region output
  static HRegion GetIcon(const HTuple &WindowHandle);
  static HRegion GetIcon(long WindowHandle);
  // Icon definition for region output.
  virtual void SetIcon(const HTuple &WindowHandle) const;
  virtual void SetIcon(long WindowHandle) const;
  // Calculate the distance between one line segment and one region.
  virtual HTuple DistanceSr(const HTuple &Row1, const HTuple &Column1, const HTuple &Row2, const HTuple &Column2, HTuple *DistanceMax) const;
  virtual HTuple DistanceSr(double Row1, double Column1, double Row2, double Column2, HTuple *DistanceMax) const;
  // Calculate the distance between one line and one region.
  virtual HTuple DistanceLr(const HTuple &Row1, const HTuple &Column1, const HTuple &Row2, const HTuple &Column2, HTuple *DistanceMax) const;
  virtual HTuple DistanceLr(double Row1, double Column1, double Row2, double Column2, HTuple *DistanceMax) const;
  // Calculate the distance between one point and one region.
  virtual HTuple DistancePr(const HTuple &Row, const HTuple &Column, HTuple *DistanceMax) const;
  virtual HTuple DistancePr(double Row, double Column, HTuple *DistanceMax) const;
  // Centres of circles for a specific radius.
  virtual HRegionArray HoughCircles(const HTuple &Radius, const HTuple &Percent, const HTuple &Mode) const;
  virtual HRegionArray HoughCircles(long Radius, long Percent, long Mode) const;
  // Return the Hough-Transform for circles with a given radius.
  virtual HImageArray HoughCircleTrans(const HTuple &Radius) const;
  virtual HImageArray HoughCircleTrans(long Radius) const;
  // Detect lines in edge images with the help of the Hough transform and
  // returns it in HNF.
  virtual HTuple HoughLines(const HTuple &AngleResolution, const HTuple &Threshold, const HTuple &AngleGap, const HTuple &DistGap, HTuple *Dist) const;
  virtual HTuple HoughLines(long AngleResolution, long Threshold, long AngleGap, long DistGap, HTuple *Dist) const;
  // Produce the Hough transform for lines within regions.
  virtual HImage HoughLineTrans(const HTuple &AngleResolution) const;
  virtual HImage HoughLineTrans(long AngleResolution) const;
  // Select those lines from a set of lines (in HNF) which
  // fit best into a region.
  virtual HRegionArray SelectMatchingLines(const HTuple &AngleIn, const HTuple &DistIn, const HTuple &LineWidth, const HTuple &Thresh, HTuple *AngleOut, HTuple *DistOut) const;
  virtual HRegionArray SelectMatchingLines(double AngleIn, double DistIn, long LineWidth, long Thresh, HTuple *AngleOut, HTuple *DistOut) const;
  // Calculate gray value moments and approximation by a first order surface
  // (plane).
  virtual double FitSurfaceFirstOrder(const HImage &Image, const HTuple &Algorithm, const HTuple &Iterations, const HTuple &ClippingFactor, double *Beta, double *Gamma) const;
  virtual double FitSurfaceFirstOrder(const HImage &Image, const char *Algorithm, long Iterations, double ClippingFactor, double *Beta, double *Gamma) const;
  // Calculate gray value moments and approximation by a second order surface.
  virtual double FitSurfaceSecondOrder(const HImage &Image, const HTuple &Algorithm, const HTuple &Iterations, const HTuple &ClippingFactor, double *Beta, double *Gamma, double *Delta, double *Epsilon, double *Zeta) const;
  virtual double FitSurfaceSecondOrder(const HImage &Image, const char *Algorithm, long Iterations, double ClippingFactor, double *Beta, double *Gamma, double *Delta, double *Epsilon, double *Zeta) const;
  // Determine a histogram of features along all threshold values.
  virtual HTuple ShapeHistoPoint(const HImage &Image, const HTuple &Feature, const HTuple &Row, const HTuple &Column, HTuple *RelativeHisto) const;
  virtual HTuple ShapeHistoPoint(const HImage &Image, const char *Feature, long Row, long Column, HTuple *RelativeHisto) const;
  // Determine a histogram of features along all threshold values.
  virtual HTuple ShapeHistoAll(const HImage &Image, const HTuple &Feature, HTuple *RelativeHisto) const;
  virtual HTuple ShapeHistoAll(const HImage &Image, const char *Feature, HTuple *RelativeHisto) const;
  // Determine the minimum and maximum gray values within regions.
  virtual double MinMaxGray(const HImage &Image, const HTuple &Percent, double *Max, double *Range) const;
  virtual double MinMaxGray(const HImage &Image, double Percent, double *Max, double *Range) const;
  // Calculate the mean and deviation of gray values.
  virtual double Intensity(const HImage &Image, double *Deviation) const;
  // Calculate the histogram of two-channel gray value images.
  virtual HImage Histo2dim(const HImage &ImageCol, const HImage &ImageRow) const;
  // Calculate the gray value distribution.
  virtual HTuple GrayHistoAbs(const HImage &Image, const HTuple &Quantization) const;
  virtual HTuple GrayHistoAbs(const HImage &Image, double Quantization) const;
  // Calculate the gray value distribution.
  virtual HTuple GrayHisto(const HImage &Image, HTuple *RelativeHisto) const;
  // Determine the entropy and anisotropy of images.
  virtual double EntropyGray(const HImage &Image, double *Anisotropy) const;
  // Calculate a co-occurrence matrix and derive gray value features thereof.
  virtual double CoocFeatureImage(const HImage &Image, const HTuple &LdGray, const HTuple &Direction, double *Correlation, double *Homogeneity, double *Contrast) const;
  virtual double CoocFeatureImage(const HImage &Image, long LdGray, long Direction, double *Correlation, double *Homogeneity, double *Contrast) const;
  // Calculate the co-occurrence matrix of a region in an image.
  virtual HImage GenCoocMatrix(const HImage &Image, const HTuple &LdGray, const HTuple &Direction) const;
  virtual HImage GenCoocMatrix(const HImage &Image, long LdGray, long Direction) const;
  // Calculate gray value moments and approximation by a plane.
  virtual double MomentsGrayPlane(const HImage &Image, double *MCol, double *Alpha, double *Beta, double *Mean) const;
  // Calculate the deviation of the gray values from the approximating
  // image plane.
  virtual double PlaneDeviation(const HImage &Image) const;
  // Compute the orientation and major axes of a region in a gray value
  // image.
  virtual double EllipticAxisGray(const HImage &Image, double *Rb, double *Phi) const;
  // Compute the area and center of gravity of a region in a gray value
  // image.
  virtual double AreaCenterGray(const HImage &Image, double *Row, double *Column) const;
  // Calculate horizontal and vertical gray-value projections.
  virtual HTuple GrayProjections(const HImage &Image, const HTuple &Mode, HTuple *VertProjection) const;
  virtual HTuple GrayProjections(const HImage &Image, const char *Mode, HTuple *VertProjection) const;
  // Geometric moments of regions.
  virtual double MomentsRegionCentralInvar(double *PSI2, double *PSI3, double *PSI4) const;
  // Geometric moments of regions.
  virtual double MomentsRegionCentral(double *I2, double *I3, double *I4) const;
  // Geometric moments of regions.
  virtual double MomentsRegion3rdInvar(double *M12, double *M03, double *M30) const;
  // Geometric moments of regions.
  virtual double MomentsRegion3rd(double *M12, double *M03, double *M30) const;
  // Smallest surrounding rectangle with any orientation.
  virtual double SmallestRectangle2(double *Column, double *Phi, double *Length1, double *Length2) const;
  // Surrounding rectangle parallel to the coordinate axes.
  virtual long SmallestRectangle1(long *Column1, long *Row2, long *Column2) const;
  // Smallest surrounding circle of a region.
  virtual double SmallestCircle(double *Column, double *Radius) const;
  // Choose regions having a certain relation to each other.
  virtual HRegion SelectShapeProto(const HRegion &Pattern, const HTuple &Feature, const HTuple &Min, const HTuple &Max) const;
  virtual HRegion SelectShapeProto(const HRegion &Pattern, const char *Feature, double Min, double Max) const;
  virtual HRegion SelectShapeProto(const HRegionArray &Pattern, const HTuple &Feature, const HTuple &Min, const HTuple &Max) const;
  virtual HRegion SelectShapeProto(const HRegionArray &Pattern, const char *Feature, double Min, double Max) const;
  // Characteristic values for runlength coding of regions.
  virtual long RunlengthFeatures(double *KFactor, double *LFactor, double *MeanLength, long *Bytes) const;
  // Search direct neighbors.
  virtual HTuple FindNeighbors(const HRegion &Regions2, const HTuple &MaxDistance, HTuple *RegionIndex2) const;
  virtual HTuple FindNeighbors(const HRegion &Regions2, long MaxDistance, HTuple *RegionIndex2) const;
  virtual HTuple FindNeighbors(const HRegionArray &Regions2, const HTuple &MaxDistance, HTuple *RegionIndex2) const;
  virtual HTuple FindNeighbors(const HRegionArray &Regions2, long MaxDistance, HTuple *RegionIndex2) const;
  // Geometric moments of regions.
  virtual double MomentsRegion2ndRelInvar(double *PHI2) const;
  // Geometric moments of regions.
  virtual double MomentsRegion2ndInvar(double *M20, double *M02) const;
  // Geometric moments of regions.
  virtual double MomentsRegion2nd(double *M20, double *M02, double *Ia, double *Ib) const;
  // Minimum distance between the contour pixels of two regions each.
  virtual double DistanceRrMin(const HRegion &Regions2, long *Row1, long *Column1, long *Row2, long *Column2) const;
  virtual double DistanceRrMin(const HRegionArray &Regions2, long *Row1, long *Column1, long *Row2, long *Column2) const;
  // Minimum distance between two regions with the help of dilatation.
  virtual long DistanceRrMinDil(const HRegion &Regions2) const;
  virtual long DistanceRrMinDil(const HRegionArray &Regions2) const;
  // Maximal distance between two boundary points of a region.
  virtual long DiameterRegion(long *Column1, long *Row2, long *Column2, double *Diameter) const;
  // Test if the region consists of the given point.
  virtual HBool TestRegionPoint(const HTuple &Row, const HTuple &Column) const;
  virtual HBool TestRegionPoint(long Row, long Column) const;
  // Select regions of a given shape.
  virtual HRegion SelectShapeStd(const HTuple &Shape, const HTuple &Percent) const;
  virtual HRegion SelectShapeStd(const char *Shape, double Percent) const;
  // Hamming distance between two regions using normalization.
  virtual long HammingDistanceNorm(const HRegion &Regions2, const HTuple &Norm, double *Similarity) const;
  virtual long HammingDistanceNorm(const HRegion &Regions2, const char *Norm, double *Similarity) const;
  virtual long HammingDistanceNorm(const HRegionArray &Regions2, const HTuple &Norm, double *Similarity) const;
  virtual long HammingDistanceNorm(const HRegionArray &Regions2, const char *Norm, double *Similarity) const;
  // Hamming distance between two regions.
  virtual long HammingDistance(const HRegion &Regions2, double *Similarity) const;
  virtual long HammingDistance(const HRegionArray &Regions2, double *Similarity) const;
  // Shape features derived from the ellipse parameters.
  virtual double Eccentricity(double *Bulkiness, double *StructureFactor) const;
  // Calculate the Euler number.
  virtual long EulerNumber(void) const;
  // Orientation of a region.
  virtual double OrientationRegion(void) const;
  // Parameters of the equivalent ellipse.
  virtual double EllipticAxis(double *Rb, double *Phi) const;
  // Pose relation of regions.
  virtual HTuple SelectRegionSpatial(const HRegion &Regions2, const HTuple &Direction, HTuple *RegionIndex2) const;
  virtual HTuple SelectRegionSpatial(const HRegion &Regions2, const char *Direction, HTuple *RegionIndex2) const;
  virtual HTuple SelectRegionSpatial(const HRegionArray &Regions2, const HTuple &Direction, HTuple *RegionIndex2) const;
  virtual HTuple SelectRegionSpatial(const HRegionArray &Regions2, const char *Direction, HTuple *RegionIndex2) const;
  // Pose relation of regions with regard to  the coordinate axes.
  virtual HTuple SpatialRelation(const HRegion &Regions2, const HTuple &Percent, HTuple *RegionIndex2, HTuple *Relation1, HTuple *Relation2) const;
  virtual HTuple SpatialRelation(const HRegion &Regions2, long Percent, HTuple *RegionIndex2, HTuple *Relation1, HTuple *Relation2) const;
  virtual HTuple SpatialRelation(const HRegionArray &Regions2, const HTuple &Percent, HTuple *RegionIndex2, HTuple *Relation1, HTuple *Relation2) const;
  virtual HTuple SpatialRelation(const HRegionArray &Regions2, long Percent, HTuple *RegionIndex2, HTuple *Relation1, HTuple *Relation2) const;
  // Shape factor for the convexity of a region.
  virtual double Convexity(void) const;
  // Contour length of a region.
  virtual double Contlength(void) const;
  // Number of connection components and holes
  virtual long ConnectAndHoles(long *NumHoles) const;
  // Shape factor for the compactness of a region.
  virtual double Compactness(void) const;
  // Shape factor for the circularity (similarity to a circle) of a region.
  virtual double Circularity(void) const;
  // Area and center of regions.
  virtual long AreaCenter(double *Row, double *Column) const;
  // Distribution of runs needed for runlength encoding of a region.
  virtual HTuple RunlengthDistribution(HTuple *Background) const;
  // Shape factors from contour.
  virtual double Roundness(double *Sigma, double *Roundness, double *Sides) const;
  // Largest inner circle of a region.
  virtual double InnerCircle(double *Column, double *Radius) const;
  // Prune the branches of a region.
  virtual HRegion Pruning(const HTuple &Length) const;
  virtual HRegion Pruning(long Length) const;
  // Reduce a region to its boundary.
  virtual HRegion Boundary(const HTuple &BoundaryType) const;
  virtual HRegion Boundary(const char *BoundaryType) const;
  // Perform a closing after an opening with multiple structuring elements.
  virtual HRegion Fitting(const HRegion &StructElements) const;
  virtual HRegion Fitting(const HRegionArray &StructElements) const;
  // Generate standard structuring elements.
  static HRegion GenStructElements(const HTuple &Type, const HTuple &Row, const HTuple &Column);
  static HRegion GenStructElements(const char *Type, long Row, long Column);
  // Reflect a region about a point.
  virtual HRegion TransposeRegion(const HTuple &Row, const HTuple &Column) const;
  virtual HRegion TransposeRegion(long Row, long Column) const;
  // Remove the result of a hit-or-miss operation from a region (sequential).
  virtual HRegion ThinningSeq(const HTuple &GolayElement, const HTuple &Iterations) const;
  virtual HRegion ThinningSeq(const char *GolayElement, long Iterations) const;
  // Remove the result of a hit-or-miss operation from a region (using a
  // Golay structuring element).
  virtual HRegion ThinningGolay(const HTuple &GolayElement, const HTuple &Rotation) const;
  virtual HRegion ThinningGolay(const char *GolayElement, long Rotation) const;
  // Remove the result of a hit-or-miss operation from a region.
  virtual HRegion Thinning(const HRegion &StructElement1, const HRegion &StructElement2, const HTuple &Row, const HTuple &Column, const HTuple &Iterations) const;
  virtual HRegion Thinning(const HRegion &StructElement1, const HRegion &StructElement2, long Row, long Column, long Iterations) const;
  // Add the result of a hit-or-miss operation to a region (sequential).
  virtual HRegion ThickeningSeq(const HTuple &GolayElement, const HTuple &Iterations) const;
  virtual HRegion ThickeningSeq(const char *GolayElement, long Iterations) const;
  // Add the result of a hit-or-miss operation to a region (using a Golay
  // structuring element).
  virtual HRegion ThickeningGolay(const HTuple &GolayElement, const HTuple &Rotation) const;
  virtual HRegion ThickeningGolay(const char *GolayElement, long Rotation) const;
  // Add the result of a hit-or-miss operation to a region.
  virtual HRegion Thickening(const HRegion &StructElement1, const HRegion &StructElement2, const HTuple &Row, const HTuple &Column, const HTuple &Iterations) const;
  virtual HRegion Thickening(const HRegion &StructElement1, const HRegion &StructElement2, long Row, long Column, long Iterations) const;
  // Hit-or-miss operation for regions using the Golay alphabet (sequential).
  virtual HRegion HitOrMissSeq(const HTuple &GolayElement) const;
  virtual HRegion HitOrMissSeq(const char *GolayElement) const;
  // Hit-or-miss operation for regions using the Golay alphabet.
  virtual HRegion HitOrMissGolay(const HTuple &GolayElement, const HTuple &Rotation) const;
  virtual HRegion HitOrMissGolay(const char *GolayElement, long Rotation) const;
  // Hit-or-miss operation for regions.
  virtual HRegion HitOrMiss(const HRegion &StructElement1, const HRegion &StructElement2, const HTuple &Row, const HTuple &Column) const;
  virtual HRegion HitOrMiss(const HRegion &StructElement1, const HRegion &StructElement2, long Row, long Column) const;
  // Generate the structuring elements of the Golay alphabet.
  static HRegion GolayElements(HRegion *StructElement2, const HTuple &GolayElement, const HTuple &Rotation, const HTuple &Row, const HTuple &Column);
  static HRegion GolayElements(HRegion *StructElement2, const char *GolayElement, long Rotation, long Row, long Column);
  // Thinning of a region.
  virtual HRegion MorphSkiz(const HTuple &Iterations1, const HTuple &Iterations2) const;
  virtual HRegion MorphSkiz(long Iterations1, long Iterations2) const;
  // Compute the morphological skeleton of a region.
  virtual HRegion MorphSkeleton(void) const;
  // Compute the union of bottom_hat and top_hat.
  virtual HRegion MorphHat(const HRegion &StructElement) const;
  // Compute the bottom hat of regions.
  virtual HRegion BottomHat(const HRegion &StructElement) const;
  // Compute the top hat of regions.
  virtual HRegion TopHat(const HRegion &StructElement) const;
  // Erode a region (using a reference point).
  virtual HRegion MinkowskiSub2(const HRegion &StructElement, const HTuple &Row, const HTuple &Column, const HTuple &Iterations) const;
  virtual HRegion MinkowskiSub2(const HRegion &StructElement, long Row, long Column, long Iterations) const;
  // Erode a region.
  virtual HRegion MinkowskiSub1(const HRegion &StructElement, const HTuple &Iterations) const;
  virtual HRegion MinkowskiSub1(const HRegion &StructElement, long Iterations) const;
  // Dilate a region (using a reference point).
  virtual HRegion MinkowskiAdd2(const HRegion &StructElement, const HTuple &Row, const HTuple &Column, const HTuple &Iterations) const;
  virtual HRegion MinkowskiAdd2(const HRegion &StructElement, long Row, long Column, long Iterations) const;
  // Perform a Minkowski addition on a region.
  virtual HRegion MinkowskiAdd1(const HRegion &StructElement, const HTuple &Iterations) const;
  virtual HRegion MinkowskiAdd1(const HRegion &StructElement, long Iterations) const;
  // Close a region with a rectangular structuring element.
  virtual HRegion ClosingRectangle1(const HTuple &Width, const HTuple &Height) const;
  virtual HRegion ClosingRectangle1(long Width, long Height) const;
  // Close a region with an element from the Golay alphabet.
  virtual HRegion ClosingGolay(const HTuple &GolayElement, const HTuple &Rotation) const;
  virtual HRegion ClosingGolay(const char *GolayElement, long Rotation) const;
  // Close a region with a circular structuring element.
  virtual HRegion ClosingCircle(const HTuple &Radius) const;
  virtual HRegion ClosingCircle(double Radius) const;
  // Close a region.
  virtual HRegion Closing(const HRegion &StructElement) const;
  // Separate overlapping regions.
  virtual HRegionArray OpeningSeg(const HRegion &StructElement) const;
  // Open a region with an element from the Golay alphabet.
  virtual HRegion OpeningGolay(const HTuple &GolayElement, const HTuple &Rotation) const;
  virtual HRegion OpeningGolay(const char *GolayElement, long Rotation) const;
  // Open a region with a rectangular structuring element.
  virtual HRegion OpeningRectangle1(const HTuple &Width, const HTuple &Height) const;
  virtual HRegion OpeningRectangle1(long Width, long Height) const;
  // Open a region with a circular structuring element.
  virtual HRegion OpeningCircle(const HTuple &Radius) const;
  virtual HRegion OpeningCircle(double Radius) const;
  // Open a region.
  virtual HRegion Opening(const HRegion &StructElement) const;
  // Erode a region sequentially.
  virtual HRegion ErosionSeq(const HTuple &GolayElement, const HTuple &Iterations) const;
  virtual HRegion ErosionSeq(const char *GolayElement, long Iterations) const;
  // Erode a region with an element from the Golay alphabet.
  virtual HRegion ErosionGolay(const HTuple &GolayElement, const HTuple &Iterations, const HTuple &Rotation) const;
  virtual HRegion ErosionGolay(const char *GolayElement, long Iterations, long Rotation) const;
  // Erode a region with a rectangular structuring element.
  virtual HRegion ErosionRectangle1(const HTuple &Width, const HTuple &Height) const;
  virtual HRegion ErosionRectangle1(long Width, long Height) const;
  // Erode a region with a circular structuring element.
  virtual HRegion ErosionCircle(const HTuple &Radius) const;
  virtual HRegion ErosionCircle(double Radius) const;
  // Erode a region (using a reference point).
  virtual HRegion Erosion2(const HRegion &StructElement, const HTuple &Row, const HTuple &Column, const HTuple &Iterations) const;
  virtual HRegion Erosion2(const HRegion &StructElement, long Row, long Column, long Iterations) const;
  // Erode a region.
  virtual HRegion Erosion1(const HRegion &StructElement, const HTuple &Iterations) const;
  virtual HRegion Erosion1(const HRegion &StructElement, long Iterations) const;
  // Dilate a region sequentially.
  virtual HRegion DilationSeq(const HTuple &GolayElement, const HTuple &Iterations) const;
  virtual HRegion DilationSeq(const char *GolayElement, long Iterations) const;
  // Dilate a region with an element from the Golay alphabet.
  virtual HRegion DilationGolay(const HTuple &GolayElement, const HTuple &Iterations, const HTuple &Rotation) const;
  virtual HRegion DilationGolay(const char *GolayElement, long Iterations, long Rotation) const;
  // Dilate a region with a rectangular structuring element.
  virtual HRegion DilationRectangle1(const HTuple &Width, const HTuple &Height) const;
  virtual HRegion DilationRectangle1(long Width, long Height) const;
  // Dilate a region with a circular structuring element.
  virtual HRegion DilationCircle(const HTuple &Radius) const;
  virtual HRegion DilationCircle(double Radius) const;
  // Dilate a region (using a reference point).
  virtual HRegion Dilation2(const HRegion &StructElement, const HTuple &Row, const HTuple &Column, const HTuple &Iterations) const;
  virtual HRegion Dilation2(const HRegion &StructElement, long Row, long Column, long Iterations) const;
  // Dilate a region.
  virtual HRegion Dilation1(const HRegion &StructElement, const HTuple &Iterations) const;
  virtual HRegion Dilation1(const HRegion &StructElement, long Iterations) const;
  // Classification of a character.
  virtual HTuple DoOcrSingle(const HImage &Image, const HTuple &OcrHandle, HTuple *Confidences) const;
  virtual HTuple DoOcrSingle(const HImage &Image, long OcrHandle, HTuple *Confidences) const;
  // Classification of characters.
  virtual HTuple DoOcrMulti(const HImage &Image, const HTuple &OcrHandle, double *Confidence) const;
  virtual HTuple DoOcrMulti(const HImage &Image, long OcrHandle, double *Confidence) const;
  // Training of an OCR classifier by the input of regions.
  virtual double TraindOcrClassBox(const HImage &Image, const HTuple &OcrHandle, const HTuple &Class) const;
  virtual double TraindOcrClassBox(const HImage &Image, long OcrHandle, const char *Class) const;
  // Storing of trained characters into a file.
  virtual void WriteOcrTrainf(const HImage &Image, const HTuple &Class, const HTuple &FileName) const;
  virtual void WriteOcrTrainf(const HImage &Image, const char *Class, const char *FileName) const;
  // Training of an OCR classifier by the input of regions.
  virtual double TestdOcrClassBox(const HImage &Image, const HTuple &OcrHandle, const HTuple &Class) const;
  virtual double TestdOcrClassBox(const HImage &Image, long OcrHandle, const char *Class) const;
  // Adding characters to a training file.
  virtual void AppendOcrTrainf(const HImage &Image, const HTuple &Class, const HTuple &FileName) const;
  virtual void AppendOcrTrainf(const HImage &Image, const char *Class, const char *FileName) const;
  // Store a polygon as a ``filled'' region.
  static HRegion GenRegionPolygonFilled(const HTuple &Rows, const HTuple &Columns);
  static HRegion GenRegionPolygonFilled(long Rows, long Columns);
  // Store a polygon as an image object.
  static HRegion GenRegionPolygon(const HTuple &Rows, const HTuple &Columns);
  static HRegion GenRegionPolygon(long Rows, long Columns);
  // Store individual pixels as image region.
  static HRegion GenRegionPoints(const HTuple &Rows, const HTuple &Columns);
  static HRegion GenRegionPoints(long Rows, long Columns);
  // Create an image region from a runlength coding.
  static HRegion GenRegionRuns(const HTuple &Row, const HTuple &ColumnBegin, const HTuple &ColumnEnd);
  static HRegion GenRegionRuns(long Row, long ColumnBegin, long ColumnEnd);
  // Create a rectangle of any orientation.
  static HRegion GenRectangle2(const HTuple &Row, const HTuple &Column, const HTuple &Phi, const HTuple &Length1, const HTuple &Length2);
  static HRegion GenRectangle2(double Row, double Column, double Phi, double Length1, double Length2);
  // Create a rectangle parallel to the coordinate axes.
  static HRegion GenRectangle1(const HTuple &Row1, const HTuple &Column1, const HTuple &Row2, const HTuple &Column2);
  static HRegion GenRectangle1(double Row1, double Column1, double Row2, double Column2);
  // Create a random region.
  static HRegion GenRandomRegion(const HTuple &Width, const HTuple &Height);
  static HRegion GenRandomRegion(long Width, long Height);
  // Create an ellipse.
  static HRegion GenEllipse(const HTuple &Row, const HTuple &Column, const HTuple &Phi, const HTuple &Radius1, const HTuple &Radius2);
  static HRegion GenEllipse(double Row, double Column, double Phi, double Radius1, double Radius2);
  // Create a circle.
  static HRegion GenCircle(const HTuple &Row, const HTuple &Column, const HTuple &Radius);
  static HRegion GenCircle(double Row, double Column, double Radius);
  // Create a checkered region.
  static HRegion GenCheckerRegion(const HTuple &WidthRegion, const HTuple &HeightRegion, const HTuple &WidthPattern, const HTuple &HeightPattern);
  static HRegion GenCheckerRegion(long WidthRegion, long HeightRegion, long WidthPattern, long HeightPattern);
  // Create a region from lines or pixels.
  static HRegion GenGridRegion(const HTuple &RowSteps, const HTuple &ColumnSteps, const HTuple &Type, const HTuple &Width, const HTuple &Height);
  static HRegion GenGridRegion(long RowSteps, long ColumnSteps, const char *Type, long Width, long Height);
  // Store input lines described in Hesse normal shape as regions.
  static HRegion GenRegionHline(const HTuple &Orientation, const HTuple &Distance);
  static HRegion GenRegionHline(double Orientation, double Distance);
  // Store input lines as regions.
  static HRegion GenRegionLine(const HTuple &BeginRow, const HTuple &BeginCol, const HTuple &EndRow, const HTuple &EndCol);
  static HRegion GenRegionLine(long BeginRow, long BeginCol, long EndRow, long EndCol);
  // Create an empty region.
  static HRegion GenEmptyRegion();
  // Test whether the regions of two objects are identical.
  virtual HBool TestEqualRegion(const HRegion &Regions2) const;
  virtual HBool TestEqualRegion(const HRegionArray &Regions2) const;
  // Paint a region in an image with a constant gray value.
  virtual HImage PaintRegion(const HImage &Image, const HTuple &Grayval, const HTuple &Type) const;
  virtual HImage PaintRegion(const HImage &Image, double Grayval, const char *Type) const;
  // Access the thickness of a region along the main axis.
  virtual HTuple GetRegionThickness(HTuple *Histogramm) const;
  // Polygon approximation of a region.
  virtual HTuple GetRegionPolygon(const HTuple &Tolerance, HTuple *Columns) const;
  virtual HTuple GetRegionPolygon(double Tolerance, HTuple *Columns) const;
  // Access the pixels of a region.
  virtual HTuple GetRegionPoints(HTuple *Columns) const;
  // Access the contour of an object.
  virtual HTuple GetRegionContour(HTuple *Columns) const;
  // Access the runlength coding of a region.
  virtual HTuple GetRegionRuns(HTuple *ColumnBegin, HTuple *ColumnEnd) const;
  // Contour of an object as chain code.
  virtual long GetRegionChain(HTuple *Column, HTuple *Chain) const;
  // Access convex hull as contour.
  virtual HTuple GetRegionConvex(HTuple *Columns) const;
  // Merge regions from line scan images.
  virtual HRegion MergeRegionsLineScan(const HRegion &PrevRegions, HRegion *PrevMergedRegions, const HTuple &ImageHeight, const HTuple &MergeBorder, const HTuple &MaxImagesRegion) const;
  virtual HRegion MergeRegionsLineScan(const HRegion &PrevRegions, HRegion *PrevMergedRegions, long ImageHeight, const char *MergeBorder, long MaxImagesRegion) const;
  virtual HRegion MergeRegionsLineScan(const HRegionArray &PrevRegions, HRegion *PrevMergedRegions, const HTuple &ImageHeight, const HTuple &MergeBorder, const HTuple &MaxImagesRegion) const;
  virtual HRegion MergeRegionsLineScan(const HRegionArray &PrevRegions, HRegion *PrevMergedRegions, long ImageHeight, const char *MergeBorder, long MaxImagesRegion) const;
  // Partition a region into rectangles of equal size.
  virtual HRegion PartitionRectangle(const HTuple &Width, const HTuple &Height) const;
  virtual HRegion PartitionRectangle(double Width, double Height) const;
  // Partition a region horizontally into rectangles.
  virtual HRegionArray PartitionDynamic(const HTuple &Distance, const HTuple &Percent) const;
  virtual HRegionArray PartitionDynamic(double Distance, double Percent) const;
  // Convert regions to a label image.
  virtual HImage RegionToLabel(const HTuple &Type, const HTuple &Width, const HTuple &Height) const;
  virtual HImage RegionToLabel(const char *Type, long Width, long Height) const;
  // Convert a region into a binary byte-image.
  virtual HImage RegionToBin(const HTuple &ForegroundGray, const HTuple &BackgroundGray, const HTuple &Width, const HTuple &Height) const;
  virtual HImage RegionToBin(long ForegroundGray, long BackgroundGray, long Width, long Height) const;
  // Return the union of two regions.
  virtual HRegion Union2(const HRegion &Region2) const;
  virtual HRegion Union2(const HRegionArray &Region2) const;
  // Compute the distance transformation of a region.
  virtual HImage DistanceTransform(const HTuple &Metric, const HTuple &Foreground, const HTuple &Width, const HTuple &Height) const;
  virtual HImage DistanceTransform(const char *Metric, const char *Foreground, long Width, long Height) const;
  // Compute the skeleton of a region.
  virtual HRegion Skeleton(void) const;
  // Apply a projective transformation to a region.
  virtual HRegion ProjectiveTransRegion(const HTuple &HomMat2D, const HTuple &Interpolation) const;
  virtual HRegion ProjectiveTransRegion(double HomMat2D, const char *Interpolation) const;
  // Apply an arbitrary affine 2D transformation to regions.
  virtual HRegion AffineTransRegion(const HTuple &HomMat2D, const HTuple &Interpolate) const;
  virtual HRegion AffineTransRegion(double HomMat2D, const char *Interpolate) const;
  // Reflect a region about the x- or y-axis.
  virtual HRegion MirrorRegion(const HTuple &RowColumn, const HTuple &WidthHeight) const;
  virtual HRegion MirrorRegion(const char *RowColumn, long WidthHeight) const;
  // Zoom a region.
  virtual HRegion ZoomRegion(const HTuple &ScaleWidth, const HTuple &ScaleHeight) const;
  virtual HRegion ZoomRegion(double ScaleWidth, double ScaleHeight) const;
  // Translate a region.
  virtual HRegion MoveRegion(const HTuple &Row, const HTuple &Column) const;
  virtual HRegion MoveRegion(long Row, long Column) const;
  // Find junctions and end points in a skeleton.
  virtual HRegion JunctionsSkeleton(HRegion *JuncPoints) const;
  // Calculate the intersection of two regions.
  virtual HRegion Intersection(const HRegion &Region2) const;
  virtual HRegion Intersection(const HRegionArray &Region2) const;
  // Partition the image plane using given regions.
  virtual HRegion Interjacent(const HTuple &Mode) const;
  virtual HRegion Interjacent(const char *Mode) const;
  // Fill up holes in regions.
  virtual HRegion FillUp(void) const;
  // Fill up holes in regions having given shape features.
  virtual HRegion FillUpShape(const HTuple &Feature, const HTuple &Min, const HTuple &Max) const;
  virtual HRegion FillUpShape(const char *Feature, double Min, double Max) const;
  // Fill gaps between regions or split overlapping regions.
  virtual HRegion ExpandRegion(const HRegion &ForbiddenArea, const HTuple &Iterations, const HTuple &Mode) const;
  virtual HRegion ExpandRegion(const HRegion &ForbiddenArea, long Iterations, const char *Mode) const;
  // Clip a region relative to its size.
  virtual HRegion ClipRegionRel(const HTuple &Top, const HTuple &Bottom, const HTuple &Left, const HTuple &Right) const;
  virtual HRegion ClipRegionRel(long Top, long Bottom, long Left, long Right) const;
  // Clip a region to a rectangle.
  virtual HRegion ClipRegion(const HTuple &Row1, const HTuple &Column1, const HTuple &Row2, const HTuple &Column2) const;
  virtual HRegion ClipRegion(long Row1, long Column1, long Row2, long Column2) const;
  // Rank operator for regions.
  virtual HRegion RankRegion(const HTuple &Width, const HTuple &Height, const HTuple &Number) const;
  virtual HRegion RankRegion(long Width, long Height, long Number) const;
  // Compute connected components of a region.
  virtual HRegionArray Connection(void) const;
  // Calculate the symmetric difference of two regions.
  virtual HRegion SymmDifference(const HRegion &Region2) const;
  virtual HRegion SymmDifference(const HRegionArray &Region2) const;
  // Calculate the difference of two regions.
  virtual HRegion Difference(const HRegion &Sub) const;
  virtual HRegion Difference(const HRegionArray &Sub) const;
  // Return the complement of a region.
  virtual HRegion Complement(void) const;
  // Determine the connected components of the background of given regions.
  virtual HRegionArray BackgroundSeg(void) const;
  // Generate a region having a given Hamming distance.
  virtual HRegion HammingChangeRegion(const HTuple &Width, const HTuple &Height, const HTuple &Distance) const;
  virtual HRegion HammingChangeRegion(long Width, long Height, long Distance) const;
  // Remove noise from a region.
  virtual HRegion RemoveNoiseRegion(const HTuple &Type) const;
  virtual HRegion RemoveNoiseRegion(const char *Type) const;
  // Transform the shape of a region.
  virtual HRegion ShapeTrans(const HTuple &Type) const;
  virtual HRegion ShapeTrans(const char *Type) const;
  // Fill gaps between regions (depending on gray value or color) or split
  // overlapping regions.
  virtual HRegion ExpandGray(const HImage &Image, const HRegion &ForbiddenArea, const HTuple &Iterations, const HTuple &Mode, const HTuple &Threshold) const;
  // Fill gaps between regions (depending on gray value or color) or split
  // overlapping regions.
  virtual HRegion ExpandGrayRef(const HImage &Image, const HRegion &ForbiddenArea, const HTuple &Iterations, const HTuple &Mode, const HTuple &RefGray, const HTuple &Threshold) const;
  // Split lines represented by one pixel wide, non-branching regions.
  virtual HRegionArray SplitSkeletonRegion(const HTuple &MaxDistance) const;
  virtual HRegionArray SplitSkeletonRegion(long MaxDistance) const;
  // Convert a histogram into a region.
  static HRegion GenRegionHisto(const HTuple &Histogram, const HTuple &Row, const HTuple &Column, const HTuple &Scale);
  static HRegion GenRegionHisto(long Histogram, long Row, long Column, long Scale);
  // Eliminate runs of a given length.
  virtual HRegion EliminateRuns(const HTuple &ElimShorter, const HTuple &ElimLonger) const;
  virtual HRegion EliminateRuns(long ElimShorter, long ElimLonger) const;
  // Construct clusters for class_ndim_norm.
  virtual HTuple LearnNdimNorm(const HRegion &Background, const HImage &Image, const HTuple &Metric, const HTuple &Distance, const HTuple &MinNumberPercent, HTuple *Center, double *Quality) const;
  virtual HTuple LearnNdimNorm(const HRegion &Background, const HImage &Image, const char *Metric, double Distance, double MinNumberPercent, HTuple *Center, double *Quality) const;
  virtual HTuple LearnNdimNorm(const HRegionArray &Background, const HImageArray &Image, const HTuple &Metric, const HTuple &Distance, const HTuple &MinNumberPercent, HTuple *Center, double *Quality) const;
  virtual HTuple LearnNdimNorm(const HRegionArray &Background, const HImageArray &Image, const char *Metric, double Distance, double MinNumberPercent, HTuple *Center, double *Quality) const;
  // Train the current classificator using a multi-channel image.
  virtual void LearnNdimBox(const HRegion &Background, const HImage &MultiChannelImage, const HTuple &ClassifHandle) const;
  virtual void LearnNdimBox(const HRegion &Background, const HImage &MultiChannelImage, long ClassifHandle) const;
  virtual void LearnNdimBox(const HRegionArray &Background, const HImageArray &MultiChannelImage, const HTuple &ClassifHandle) const;
  virtual void LearnNdimBox(const HRegionArray &Background, const HImageArray &MultiChannelImage, long ClassifHandle) const;
  // Receive regions over a socket connection.
  static HRegion ReceiveRegion(const HTuple &Socket);
  static HRegion ReceiveRegion(long Socket);
  // Send regions over a socket connection.
  virtual void SendRegion(const HTuple &Socket) const;
  virtual void SendRegion(long Socket) const;
  // Generate XLD contours from regions.
  virtual HXLDCont GenContourRegionXld(const HTuple &Mode) const;
  virtual HXLDCont GenContourRegionXld(const char *Mode) const;
  // Convert a skeleton into XLD contours.
  virtual HXLDContArray GenContoursSkeletonXld(const HTuple &Length, const HTuple &Mode) const;
  virtual HXLDContArray GenContoursSkeletonXld(long Length, const char *Mode) const;
};

#endif
